function filtercell(el, set, action) {
	switch (action) {
		case 'focus':
			el.classList[set ? 'add' : 'remove'](action);
			break;
		case 'target':
			if (set) el.classList.toggle(action);
			break;
		case 'remove':
			if (set) el.style.display = 'none';
			break;
		case 'add':
			if (set) el.style.display = '';
			break;
		case 'toggle':
			if (set) el.style.display = el.style.display == 'none' ? '' : 'none';
			break;
		case 'filter':
			el.style.display = set ? '' : 'none';
			if (!Element.prototype.hasOwnProperty('classList')) return;
			el.classList.remove('focus');
			break;
		default: // reset
			el.classList.remove('focus');
			el.classList.remove('target');
	}
}

function filterrows(table, match, action) {
	var rows = table.tBodies[0].rows;
	for (var i = 0; i < rows.length; i++) {
		filtercell(rows[i], match && match(rows[i]), action);
	}
}

function filtertable(query, action) {
	var table = document.getElementsByTagName('TABLE')[0];

	if (!action) {
		var match = /^([-+?=]?)(.*)/.exec(query);
		switch (match[1]) {
			case '+': action = 'add';    break;
			case '-': action = 'remove'; break;
			case '?': action = 'toggle'; break;
			case '=': action = 'filter'; break;
		}
		query = match[2];
	}

	if (/^[A-Z0-9 ]{2,}$/.test(query)) {
		// category title if all uppercase
		var match = function(row) {
			return row.cells[0].title.match(query, 'i');
		};
	}
	else if (action == 'focus' && query.length <= 1) {
		// prevent superfluous highlighting
		var match = false;
	}
	else {
		// title text (case-insensitive unless caps in input)
		var match = function(row) {
			return row.cells[1].textContent.match(query, /[A-Z]/.test(query) ? '' : 'i');
		};
	}
	filterrows(table, match, action || 'filter');
}

function newelement(tagname, attrlist, childlist) {
	if (!attrlist) return document.createTextNode(tagname);
	var el = document.createElement(tagname);
	for (var name in attrlist)
		el.setAttribute(name, attrlist[name]);
	if (childlist) for (var i = 0; i < childlist.length; i++)
		el.appendChild(childlist[i]);
	return el;
}

function prependsearch(target) {
	target.parentNode.insertBefore(newelement(
		'form', {
			id: 'search',
			'class': 'aside',
			onsubmit: "filtertable(this.q.value); this.q.value = ''; return false",
		},
		[
			newelement('input', {
				type: 'search',
				name: 'q',
				onkeyup: "filtertable(this.value, 'focus')",
			}),
			newelement('input', {
				type: 'button',
				value: 'toggle',
				onclick: "filtertable(this.form.q.value, 'target')",
			}),
			newelement('input', {type:'submit', value:'filter'}),
		]
	), target);
}

